@@ -24,6 +24,15 @@
     [super dealloc];
 }
 
+- (id)init
+{
+	if (self = [super init])
+	{
+		iconImage = nil;
+	}
+	return self;
+}
+
 - (void)setAttributedStringValueFromFileSystemBrowserNode:(FileSystemBrowserNode*)node {
 	// Given a particular FSNodeInfo object set up our display properties.
 	NSString *stringValue = [node lastPathComponent];
@@ -111,6 +120,13 @@
   }
 }
 
+// disable the toopltip support - it crashes on Leopard because we don't link against the new library.  Not sure why though...
+- (NSRect)expansionFrameWithFrame:(NSRect)cellFrame inView:(NSView *)view {
+  NSRect r;
+  memset(&r, 0, sizeof(r));
+  return r;
+}
+
 @end
 
 @implementation FileSystemBrowserCell (PrivateUtilities)
