@@ -64,7 +64,12 @@
 
 - (NSArray *)subNodes {
     NSString       *subNodePath = nil;
-    NSEnumerator   *subNodePaths = [[[NSFileManager defaultManager] directoryContentsAtPath: [self absolutePath]] objectEnumerator];
+		NSArray				 *subNodePathsArray = [[NSFileManager defaultManager] directoryContentsAtPath: [self absolutePath]];
+		
+		// sort them
+		subNodePathsArray = [subNodePathsArray sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
+		
+    NSEnumerator   *subNodePaths = [subNodePathsArray objectEnumerator];
     NSMutableArray *subNodes = [NSMutableArray array];
     
     while ((subNodePath=[subNodePaths nextObject])) {
