@@ -102,6 +102,14 @@
 		[track setTrackNumber:[tag getTrack]];
 		[track setYear:[tag getYear]];
 		
+    NSMutableArray *images = [tag getImage];
+    if ([images count] > 0)
+    {
+      NSDictionary *dict = [images objectAtIndex:0];
+      NSBitmapImageRep *image = [dict objectForKey:@"Image"];
+      [track setImage:image];
+    }
+    
 		[tag release];
 		
 		// read the length from the header (use this as it is faster than id3.framework
