@@ -285,6 +285,10 @@
 	{
 		[self enableTransfers];
 		[self showItemCounts];
+    
+    if ([aNotification object] == iTunesTable)
+      [self showDrawerInfo];
+    
 		return;
 	}
 	[super tableViewSelectionDidChange:aNotification];
@@ -509,25 +513,30 @@
 - (void)showDrawerInfo
 {
 	// update display to show new track
-	if ([njbTable selectedRow] == -1)
-	{
-		[drawerController clearAll];
-		[drawerController disableAll];
-	}
-	else
-	{
-		[drawerController showTrack:(Track *)[[arrayController arrangedObjects] objectAtIndex:[njbTable selectedRow]]];
+  if (activeObject == OBJECT_NJB_TABLE && [njbTable selectedRow] > -1)
+  {
+    [drawerController showTrack:(Track *)[[arrayController arrangedObjects] objectAtIndex:[njbTable selectedRow]]];
 		if (![theNJB isConnected])
 			[drawerController disableWrite];
+  }
+  else if ([iTunesTable selectedRow] > -1)
+  {
+    [drawerController showTrack:(Track *)[[iTunesArrayController arrangedObjects] objectAtIndex:[iTunesTable selectedRow]]];
+    [drawerController disableWrite];
+  }
+  else
+  {
+		[drawerController clearAll];
+		[drawerController disableAll];
 	}
 }
 
 - (void)loadiTunesLibrary
 {
 	[iTunesArrayController removeAll];
-	
-	NSString *iTunesLibrary = [preferences iTunesMusicLibraryLocation];
-	iTunesLibrary = [iTunesLibrary stringByExpandingTildeInPath];
+  
+  NSString *iTunesDirectory = [[preferences iTunesDirectory] stringByExpandingTildeInPath];
+	NSString *iTunesLibrary = [NSString stringWithFormat:@"%@/iTunes Music Library.xml", iTunesDirectory];
 	NSDictionary *iTunesDict = [NSDictionary dictionaryWithContentsOfFile:iTunesLibrary];
 	
 	// if we made an error loading it then give up
@@ -539,8 +548,54 @@
 										nil, nil);
 		return;
 	}
-	
+  
+  /* The album art is complicated here.  iTunes has a directory with itc files in
+    iTunes/Album Artwork/.  The filenames are of the form libraryPersistentID-TrackPersistentID.itc.
+    So here we go through the filesystem and map track IDs to filenames.  Then we go through all the tracks
+    and find the itc filename for that track.  Note that the artwork is only stored for one track (I think
+    normally track 1) so we have to build up another map from album name to itc.
+  */
+  
+  // firstly make a dictionary of album art itc file locations
+  NSString *libraryPersistentID = [iTunesDict objectForKey:@"Library Persistent ID"];
+  NSMutableDictionary *itcFileDict = [[NSMutableDictionary alloc] init];
+  NSArray *dirsToSearch = [NSArray arrayWithObjects:[NSString stringWithFormat:@"%@/Album Artwork/Local/%@", iTunesDirectory, libraryPersistentID],
+    [NSString stringWithFormat:@"%@/Album Artwork/Downloaded/%@", iTunesDirectory, libraryPersistentID],
+    [NSString stringWithFormat:@"%@/Album Artwork/Cache/%@", iTunesDirectory, libraryPersistentID], nil];
+  NSEnumerator *baseDirEnum = [dirsToSearch objectEnumerator];
+  NSString *baseDir = nil;
+  while (baseDir = [baseDirEnum nextObject])
+  {
+    NSDirectoryEnumerator *dirEnum = [[NSFileManager defaultManager] enumeratorAtPath:baseDir];
+    NSString *file;
+    while (file = [dirEnum nextObject])
+    {
+      if ([[file pathExtension] isEqualToString: @"itc"])
+      {
+        //NSLog([NSString stringWithFormat:@"got itc file: %@", file]);
+      
+        // filename should be in the form of libraryid-itemid.itc
+        NSArray *pathComponents = [[[file lastPathComponent] stringByDeletingPathExtension] componentsSeparatedByString:@"-"];
+        if (![libraryPersistentID isEqualToString:[pathComponents objectAtIndex:0]])
+          NSLog([NSString stringWithFormat:@"Invalid filename for iTunes albumart: %@", file]);
+        else
+        {
+          //NSLog([NSString stringWithFormat:@"persistent id: %@", [pathComponents objectAtIndex:1]]);
+      
+          [itcFileDict setObject:[NSString stringWithFormat:@"%@/%@", baseDir, file] forKey:[pathComponents objectAtIndex:1]];
+        }
+      }
+    }
+  }
+
+  // to store the map from album name to itc file
+  NSMutableDictionary *itcFileDictFromAlbums = [[NSMutableDictionary alloc] init];
+  
+	// Now do the tracks
 	NSDictionary *tracksDict = [iTunesDict objectForKey:@"Tracks"];
+  
+  // to store the array since we can't enumerate from array controller without sorting
+  NSMutableArray *iTunesTracks = [[NSMutableArray alloc] init];
 	
 	NSDictionary *thisTrackDict;
 	NSArray *tracksDictArray = [tracksDict allValues];
@@ -570,10 +625,33 @@
 		[track setFilesize:[[thisTrackDict valueForKey:@"Size"] unsignedLongLongValue]];
     [track setDateAdded:[thisTrackDict valueForKey:@"Date Added"]];
     [track setRating:[[thisTrackDict valueForKey:@"Rating"] unsignedIntValue]];
+
+    NSString *itcPath = [itcFileDict objectForKey:[thisTrackDict objectForKey:@"Persistent ID"]];
+    if (itcPath != nil)
+    {
+      [itcFileDictFromAlbums setObject:itcPath forKey:[track album]];
+      [track setItcFilePath:itcPath];
+    }    
 			
 		[iTunesArrayController addObject:track];
+    [iTunesTracks addObject:track];
 		[track release];
 	}
+  
+  // now fill in artwork from albums (can't do before since tracks may come in the wrong order)
+  objectEnumerator = [iTunesTracks objectEnumerator];
+  Track *curTrack = nil;
+  while (curTrack = [objectEnumerator nextObject])
+  {
+    if ([[curTrack itcFilePath] isEqualToString:@""])
+    {
+      [curTrack setItcFilePath:[itcFileDictFromAlbums objectForKey:[curTrack album]]];
+    }
+  }
+  
+  [itcFileDict release];
+  [itcFileDictFromAlbums release];
+  [iTunesTracks release];
 	
 	[self showItemCounts];
 }
