@@ -241,13 +241,6 @@ StatusDisplayer *statusDisplayerGlobal;
   }
 }
 
-/*
-//TEST need for objectproplist debug in connect
-#include "/Users/richard/njb/XNJB/libmtp/src/ptp-pack.c"
-#include <fcntl.h>
-//!TEST
-*/
-
 /* connect to the NJB
  */
 - (NJBTransactionResult *)connect
@@ -791,6 +784,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		if (trackmeta->filename != NULL)
 			free(trackmeta->filename);
 		trackmeta->filename = strdup(lastPathComponent);
+    trackmeta->parent_id = [self musicAssoc];
       
     // clear any errors we don't want so we can see if we get a cancellation error below
     LIBMTP_Clear_Errorstack(device);
@@ -801,7 +795,7 @@ StatusDisplayer *statusDisplayerGlobal;
     cancelCurrentTransaction = NO;
     progressParams.cancelCurrentTransaction = &cancelCurrentTransaction;
     
-		if (LIBMTP_Send_Track_From_File(device, filename, trackmeta, mtp_progress, &progressParams, [self musicAssoc]) != 0)
+		if (LIBMTP_Send_Track_From_File(device, filename, trackmeta, mtp_progress, &progressParams) != 0)
 		{
       LIBMTP_error_t *errorStack = LIBMTP_Get_Errorstack(device);
       if (errorStack != NULL)
@@ -841,8 +835,9 @@ StatusDisplayer *statusDisplayerGlobal;
 	[self updateDiskSpace];
 	
 	// update my cache
-  // TOOD: should we do this?
+  // TODO: should we do this?
   [track setImage:nil];
+  [track setItcFilePath:@""];
 	[cachedTrackList addObject:track];
 	// tell others the track list has changed
 	[statusDisplayer postNotificationName:NOTIFICATION_TRACK_LIST_MODIFIED object:self];
@@ -1362,6 +1357,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		genfile->filetype = objectFormat;
 		// todo: safe?
 		genfile->filename = strdup([[[dataFile fullPath] lastPathComponent] fileSystemRepresentation]);
+    genfile->parent_id = parenthandle;
 		
     // clear any errors we don't want so we can see if we get a cancellation error below
     LIBMTP_Clear_Errorstack(device);
@@ -1372,7 +1368,7 @@ StatusDisplayer *statusDisplayerGlobal;
     cancelCurrentTransaction = NO;
     progressParams.cancelCurrentTransaction = &cancelCurrentTransaction;
     
-		if (LIBMTP_Send_File_From_File(device, [[dataFile fullPath] fileSystemRepresentation], genfile, mtp_progress, &progressParams, parenthandle) != 0)
+		if (LIBMTP_Send_File_From_File(device, [[dataFile fullPath] fileSystemRepresentation], genfile, mtp_progress, &progressParams) != 0)
 		{
       LIBMTP_error_t *errorStack = LIBMTP_Get_Errorstack(device);
       if (errorStack != NULL)
@@ -1435,8 +1431,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		if (device->storage == NULL)
 			return;
 		
-		totalDiskSpace = device->storage[0].MaxCapacity;
-		freeDiskSpace = device->storage[0].FreeSpaceInBytes;
+		totalDiskSpace = device->storage->MaxCapacity;
+		freeDiskSpace = device->storage->FreeSpaceInBytes;
 		
 		[statusDisplayer updateDiskSpace:totalDiskSpace withFreeSpace:freeDiskSpace];
 	}
@@ -1700,6 +1696,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		libmtpPlaylist->playlist_id = [playlist playlistID];
 		libmtpPlaylist->no_tracks = size;
 		libmtpPlaylist->tracks = trackArray;
+    libmtpPlaylist->parent_id = device->default_playlist_folder;
 		
 		int playlistState = [playlist state];
 		[playlist setState:NJB_PL_UNCHANGED];
@@ -1707,7 +1704,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		if (playlistState == NJB_PL_NEW)
 		{
-			if (LIBMTP_Create_New_Playlist(device, libmtpPlaylist, device->default_playlist_folder) != 0)
+			if (LIBMTP_Create_New_Playlist(device, libmtpPlaylist) != 0)
 			{
 				LIBMTP_destroy_playlist_t(libmtpPlaylist);
 				NSString *errorString = [self njbErrorString];
@@ -1909,7 +1906,7 @@ int mtp_progress(uint64_t const sent, uint64_t const total, void const * const d
 	}
 	else
 	{
-		unsigned newDirID = LIBMTP_Create_Folder(device, [[dir name] UTF8String], [parentDir itemID]);
+		unsigned newDirID = LIBMTP_Create_Folder(device, [[dir name] UTF8String], [parentDir itemID], 0);
 		if (newDirID == 0)
 		{
       NSString *errorString = [self njbErrorString];
@@ -2247,13 +2244,14 @@ int mtp_progress(uint64_t const sent, uint64_t const total, void const * const d
   }
   
   LIBMTP_album_t *libmtpAlbum = [album toLIBMTPAlbum:track];
+  libmtpAlbum->parent_id = device->default_album_folder;
 	
   int playlistState = [album state];
   [album setState:NJB_PL_UNCHANGED];
 		
   if (playlistState == NJB_PL_NEW)
   {
-    if (LIBMTP_Create_New_Album(device, libmtpAlbum, device->default_album_folder) != 0)
+    if (LIBMTP_Create_New_Album(device, libmtpAlbum) != 0)
     {
       LIBMTP_destroy_album_t(libmtpAlbum);
       NSLog(@"Could not create new album");
