@@ -29,8 +29,8 @@
 #ifndef LIBMTP_H_INCLUSION_GUARD
 #define LIBMTP_H_INCLUSION_GUARD
 
-#define LIBMTP_VERSION 0.2.6.1
-#define LIBMTP_VERSION_STRING "0.2.6.1"
+#define LIBMTP_VERSION 0.3.0
+#define LIBMTP_VERSION_STRING "0.3.0"
 
 /* This handles MSVC pecularities */
 #ifdef _MSC_VER
@@ -236,6 +236,7 @@ struct LIBMTP_mtpdevice_struct {
    * The storage for this device, do not use strings in here without 
    * copying them first, and beware that this list may be rebuilt at
    * any time.
+   * @see LIBMTP_Get_Storage()
    */
   LIBMTP_devicestorage_t *storage;
   /**
@@ -274,6 +275,7 @@ struct LIBMTP_mtpdevice_struct {
 struct LIBMTP_file_struct {
   uint32_t item_id; /**< Unique item ID */
   uint32_t parent_id; /**< ID of parent folder */
+  uint32_t storage_id; /**< ID of storage holding this file */
   char *filename; /**< Filename of this file */
   uint64_t filesize; /**< Size of file in bytes */
   LIBMTP_filetype_t filetype; /**< Filetype used for the current file */
@@ -286,6 +288,7 @@ struct LIBMTP_file_struct {
 struct LIBMTP_track_struct {
   uint32_t item_id; /**< Unique item ID */
   uint32_t parent_id; /**< ID of parent folder */
+  uint32_t storage_id; /**< ID of storage holding this track */
   char *title; /**< Track title */
   char *artist; /**< Name of recording artist */
   char *composer; /**< Name of recording composer */
@@ -313,6 +316,7 @@ struct LIBMTP_track_struct {
 struct LIBMTP_playlist_struct {
   uint32_t playlist_id; /**< Unique playlist ID */
   uint32_t parent_id; /**< ID of parent folder */
+  uint32_t storage_id; /**< ID of storage holding this playlist */
   char *name; /**< Name of playlist */
   uint32_t *tracks; /**< The tracks in this playlist */
   uint32_t no_tracks; /**< The number of tracks in this playlist */
@@ -325,6 +329,7 @@ struct LIBMTP_playlist_struct {
 struct LIBMTP_album_struct {
   uint32_t album_id; /**< Unique playlist ID */
   uint32_t parent_id; /**< ID of parent folder */
+  uint32_t storage_id; /**< ID of storage holding this album */
   char *name; /**< Name of album */
   char *artist; /**< Name of album artist */
   char *composer; /**< Name of recording composer */
@@ -340,6 +345,7 @@ struct LIBMTP_album_struct {
 struct LIBMTP_folder_struct {
   uint32_t folder_id; /**< Unique folder ID */
   uint32_t parent_id; /**< ID of parent folder */
+  uint32_t storage_id; /**< ID of storage holding this folder */
   char *name; /**< Name of folder */
   LIBMTP_folder_t *sibling; /**< Next folder at same level or NULL if no more */
   LIBMTP_folder_t *child; /**< Child folder or NULL if no children */
@@ -393,11 +399,14 @@ int LIBMTP_Get_Supported_Devices_List(LIBMTP_device_entry_t ** const, int * cons
  * @defgroup basic The basic device management API.
  * @{
  */
-int LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t **, int *);
+LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t **, int *);
+LIBMTP_mtpdevice_t *LIBMTP_Open_Raw_Device(LIBMTP_raw_device_t *);
+/* Begin old, legacy interface */
 LIBMTP_mtpdevice_t *LIBMTP_Get_First_Device(void);
 LIBMTP_error_number_t LIBMTP_Get_Connected_Devices(LIBMTP_mtpdevice_t **);
 uint32_t LIBMTP_Number_Devices_In_List(LIBMTP_mtpdevice_t *);
 void LIBMTP_Release_Device_List(LIBMTP_mtpdevice_t*);
+/* End old, legacy interface */
 void LIBMTP_Release_Device(LIBMTP_mtpdevice_t*);
 void LIBMTP_Dump_Device_Info(LIBMTP_mtpdevice_t*);
 int LIBMTP_Reset_Device(LIBMTP_mtpdevice_t*);
@@ -445,10 +454,10 @@ int LIBMTP_Get_File_To_File_Descriptor(LIBMTP_mtpdevice_t*, uint32_t const, int
 			LIBMTP_progressfunc_t const, void const * const);
 int LIBMTP_Send_File_From_File(LIBMTP_mtpdevice_t *, char const * const,
 	                 LIBMTP_file_t * const, LIBMTP_progressfunc_t const,
-			 void const * const, uint32_t const);
+			 void const * const);
 int LIBMTP_Send_File_From_File_Descriptor(LIBMTP_mtpdevice_t *, int const,
 	                LIBMTP_file_t * const, LIBMTP_progressfunc_t const,
-			void const * const, uint32_t const);
+			void const * const);
 LIBMTP_filesampledata_t *LIBMTP_new_filesampledata_t(void);
 void LIBMTP_destroy_filesampledata_t(LIBMTP_filesampledata_t *);
 int LIBMTP_Get_Representative_Sample_Format(LIBMTP_mtpdevice_t *,
@@ -477,11 +486,11 @@ int LIBMTP_Get_Track_To_File_Descriptor(LIBMTP_mtpdevice_t*, uint32_t const, int
 int LIBMTP_Send_Track_From_File(LIBMTP_mtpdevice_t *,
 			 char const * const, LIBMTP_track_t * const,
                          LIBMTP_progressfunc_t const,
-			 void const * const, uint32_t const);
+			 void const * const);
 int LIBMTP_Send_Track_From_File_Descriptor(LIBMTP_mtpdevice_t *,
 			 int const, LIBMTP_track_t * const,
                          LIBMTP_progressfunc_t const,
-			 void const * const, uint32_t const);
+			 void const * const);
 int LIBMTP_Update_Track_Metadata(LIBMTP_mtpdevice_t *,
 			LIBMTP_track_t const * const);
 int LIBMTP_Track_Exists(LIBMTP_mtpdevice_t *, uint32_t);
@@ -496,7 +505,7 @@ LIBMTP_folder_t *LIBMTP_new_folder_t(void);
 void LIBMTP_destroy_folder_t(LIBMTP_folder_t*);
 LIBMTP_folder_t *LIBMTP_Get_Folder_List(LIBMTP_mtpdevice_t*);
 LIBMTP_folder_t *LIBMTP_Find_Folder(LIBMTP_folder_t*, uint32_t const);
-uint32_t LIBMTP_Create_Folder(LIBMTP_mtpdevice_t*, char *, uint32_t);
+uint32_t LIBMTP_Create_Folder(LIBMTP_mtpdevice_t*, char *, uint32_t, uint32_t);
 /** @} */
 
 
@@ -509,7 +518,7 @@ LIBMTP_playlist_t *LIBMTP_new_playlist_t(void);
 void LIBMTP_destroy_playlist_t(LIBMTP_playlist_t *);
 LIBMTP_playlist_t *LIBMTP_Get_Playlist_List(LIBMTP_mtpdevice_t *);
 LIBMTP_playlist_t *LIBMTP_Get_Playlist(LIBMTP_mtpdevice_t *, uint32_t const);
-int LIBMTP_Create_New_Playlist(LIBMTP_mtpdevice_t *, LIBMTP_playlist_t * const, uint32_t const);
+int LIBMTP_Create_New_Playlist(LIBMTP_mtpdevice_t *, LIBMTP_playlist_t * const);
 int LIBMTP_Update_Playlist(LIBMTP_mtpdevice_t *, LIBMTP_playlist_t const * const);
 
 /**
@@ -521,7 +530,7 @@ LIBMTP_album_t *LIBMTP_new_album_t(void);
 void LIBMTP_destroy_album_t(LIBMTP_album_t *);
 LIBMTP_album_t *LIBMTP_Get_Album_List(LIBMTP_mtpdevice_t *);
 LIBMTP_album_t *LIBMTP_Get_Album(LIBMTP_mtpdevice_t *, uint32_t const);
-int LIBMTP_Create_New_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t * const, uint32_t const);
+int LIBMTP_Create_New_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t * const);
 int LIBMTP_Update_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t const * const);
 
 /**
