@@ -1,20 +1,8 @@
 /*
  * Darwin/MacOS X Support
  *
- * (c) 2002-2005 Nathan Hjelm <hjelmn@users.sourceforge.net>
+ * (c) 2002-2004 Nathan Hjelm <hjelmn@users.sourceforge.net>
  *
- * (04/17/2005):
- *   - Lots of minor fixes.
- *   - Endpoint table now made by claim_interface to fix a bug.
- *   - Merged Read/Write to make modifications easier.
- * (03/25/2005):
- *   - Fixed a bug when using asynchronous callbacks within a multi-threaded application.
- * (03/14/2005):
- *   - Added an endpoint table to speed up bulk transfers.
- * 0.1.11 (02/22/2005):
- *   - Updated error checking in read/write routines to check completion codes.
- *   - Updated set_configuration so that the open interface is reclaimed before completion.
- *   - Fixed several typos.
  * 0.1.8 (01/12/2004):
  *   - Fixed several memory leaks.
  *   - Readded 10.0 support
@@ -41,6 +29,8 @@
  * 0.1a  (01/03/2002):
  *   - Driver mostly completed using the macosx driver I wrote for my rioutil software.
  *
+ * this driver is EXPERIMENTAL, use at your own risk and e-mail me bug reports.
+ *
  * Derived from Linux version by Richard Tobin.
  * Also partly derived from BSD version.
  *
@@ -65,60 +55,63 @@
 
 /* some defines */
 /* IOUSBInterfaceInferface */
+/* 10.4 defines copied over from CVS by RAL */
+/* since we need to work for OS X 10.3.9 we will force to use
+ * the 10.3 versions */
+
 /*#if defined (kIOUSBInterfaceInterfaceID220)
 
-// #warning "libusb being compiled for 10.4 or later"
+#warning "libusb being compiled for 10.4 or later"
 #define usb_interface_t IOUSBInterfaceInterface220
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID220
-#define InterfaceVersion 220*/
+#define InterfaceVersion 220
 
-//#elif defined (kIOUSBInterfaceInterfaceID197)
-#if defined (kIOUSBInterfaceInterfaceID197)
+#elif defined (kIOUSBInterfaceInterfaceID197)*/
 
-// #warning "libusb being compiled for 10.3 or later"
+#warning "libusb being compiled for 10.3 or higher"
 #define usb_interface_t IOUSBInterfaceInterface197
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID197
 #define InterfaceVersion 197
 
-#elif defined (kIOUSBInterfaceInterfaceID190)
+/*#elif defined (IOUSBINTERFACE_FUNCS_190)
 
-// #warning "libusb being compiled for 10.2 or later"
+#warning "libusb being compiled for 10.2 or higher"
 #define usb_interface_t IOUSBInterfaceInterface190
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID190
 #define InterfaceVersion 190
 
-#elif defined (kIOUSBInterfaceInterfaceID182)
+#elif defined (IOUSBINTERFACE_FUNCS_182)
 
-// #warning "libusb being compiled for 10.1 or later"
+#warning "libusb being compiled for 10.1 or higher"
 #define usb_interface_t IOUSBInterfaceInterface182
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID182
 #define InterfaceVersion 182
 
 #else
 
-/* No timeout functions available! Time to upgrade your os. */
-#warning "libusb being compiled without support for timeout bulk functions! 10.0 and up"
+/ * No timeout functions available! Time to upgrade your os. * /
+#warning "libusb being compiled without support for timout bulk functions! 10.0 and up"
 #define usb_interface_t IOUSBInterfaceInterface
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID
 #define LIBUSB_NO_TIMEOUT_INTERFACE
 #define InterfaceVersion 180
 
-#endif
+#endif */
 
 /* IOUSBDeviceInterface */
-#if defined (kIOUSBDeviceInterfaceID197)
+#if defined (IOUSBDEVICE_FUNCS_197)
 
 #define usb_device_t    IOUSBDeviceInterface197
 #define DeviceInterfaceID kIOUSBDeviceInterfaceID197
 #define DeviceVersion 197
 
-#elif defined (kIOUSBDeviceInterfaceID187)
+#elif defined (IOUSBDEVICE_FUNCS_187)
 
 #define usb_device_t    IOUSBDeviceInterface187
 #define DeviceInterfaceID kIOUSBDeviceInterfaceID187
 #define DeviceVersion 187
 
-#elif defined (kIOUSBDeviceInterfaceID182)
+#elif defined (IOUSBDEVICE_FUNCS_182)
 
 #define usb_device_t    IOUSBDeviceInterface182
 #define DeviceInterfaceID kIOUSBDeviceInterfaceID182
@@ -137,25 +130,12 @@
 typedef IOReturn io_return_t;
 typedef IOCFPlugInInterface *io_cf_plugin_ref_t;
 typedef SInt32 s_int32_t;
-typedef IOReturn (*rw_async_func_t)(void *self, UInt8 pipeRef, void *buf, UInt32 size,
-				    IOAsyncCallback1 callback, void *refcon);
-typedef IOReturn (*rw_async_to_func_t)(void *self, UInt8 pipeRef, void *buf, UInt32 size,
-				       UInt32 noDataTimeout, UInt32 completionTimeout,
-				       IOAsyncCallback1 callback, void *refcon);
-
-#if !defined(IO_OBJECT_NULL)
-#define IO_OBJECT_NULL ((io_object_t)0)
-#endif
 
 /* Darwin/OS X impl does not use fd field, instead it uses this */
 struct darwin_dev_handle {
   usb_device_t **device;
   usb_interface_t **interface;
   int open;
-
-  /* stored translation table for pipes to endpoints */
-  int num_endpoints;
-  unsigned char *endpoint_addrs;
 };
 
 static CFMutableDictionaryRef matchingDict;
@@ -177,19 +157,15 @@ static char *darwin_error_str (int result) {
   case kIOReturnNoDevice:
     return "no connection to an IOService";
   case kIOUSBNoAsyncPortErr:
-    return "no async port has been opened for interface";
+    return "no asyc port has been opened for interface";
   case kIOReturnExclusiveAccess:
     return "another process has device opened for exclusive access";
   case kIOUSBPipeStalled:
     return "pipe is stalled";
   case kIOReturnError:
-    return "could not establish a connection to the Darwin kernel";
-  case kIOUSBTransactionTimeout:
-    return "transaction timed out";
+    return "could not establish a connection to Darin kernel";
   case kIOReturnBadArgument:
     return "invalid argument";
-  case kIOReturnAborted:
-    return "transaction aborted";
   default:
     return "unknown error";
   }
@@ -213,82 +189,31 @@ static int darwin_to_errno (int result) {
     return LUSBDARWINSTALL;
   case kIOReturnBadArgument:
     return EINVAL;
-  case kIOUSBTransactionTimeout:
-    return ETIMEDOUT;
   case kIOReturnError:
   default:
     return 1;
   }
 }
 
-static int usb_setup_iterator (io_iterator_t *deviceIterator)
-{
-  int result;
-
-  /* set up the matching dictionary for class IOUSBDevice and its subclasses.
-     It will be consumed by the IOServiceGetMatchingServices call */
-  if ((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL) {
-    darwin_cleanup ();
-    
-    USB_ERROR_STR(-1, "libusb/darwin.c usb_setup_iterator: Could not create a matching dictionary.\n");
-  }
-
-  result = IOServiceGetMatchingServices(masterPort, matchingDict, deviceIterator);
-  matchingDict = NULL;
-
-  if (result != kIOReturnSuccess)
-    USB_ERROR_STR (-darwin_to_errno (result), "libusb/darwin.c usb_setup_iterator: IOServiceGetMatchingServices: %s\n",
-		   darwin_error_str(result));
-
-  return 0;
-}
-
-static usb_device_t **usb_get_next_device (io_iterator_t deviceIterator, UInt32 *locationp)
-{
-  io_cf_plugin_ref_t *plugInInterface = NULL;
-  usb_device_t **device;
-  io_service_t usbDevice;
-  long result, score;
-
-  if (!IOIteratorIsValid (deviceIterator) || !(usbDevice = IOIteratorNext(deviceIterator)))
-    return NULL;
-  
-  result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
-					     kIOCFPlugInInterfaceID, &plugInInterface,
-					     &score);
-  
-  result = IOObjectRelease(usbDevice);
-  if (result || !plugInInterface)
-    return NULL;
-  
-  (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(DeviceInterfaceID),
-				     (LPVOID)&device);
-  
-  (*plugInInterface)->Stop(plugInInterface);
-  IODestroyPlugInInterface (plugInInterface);
-  plugInInterface = NULL;
-  
-  (*(device))->GetLocationID(device, locationp);
-
-  return device;
-}
-
 int usb_os_open(usb_dev_handle *dev)
 {
   struct darwin_dev_handle *device;
 
   io_return_t result;
   io_iterator_t deviceIterator;
+  io_service_t usbDevice;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
 
   usb_device_t **darwin_device;
 
-  UInt32 location = *((UInt32 *)dev->device->dev);
-  UInt32 dlocation;
+  s_int32_t score;
+  u_int32_t location = *((u_int32_t *)dev->device->dev);
+  u_int32_t dlocation;
 
   if (!dev)
     USB_ERROR(-ENXIO);
 
-  if (masterPort == MACH_PORT_NULL)
+  if (!masterPort)
     USB_ERROR(-EINVAL);
 
   device = calloc(1, sizeof(struct darwin_dev_handle));
@@ -300,19 +225,52 @@ int usb_os_open(usb_dev_handle *dev)
 	    dev->device->descriptor.idVendor,
 	    dev->device->descriptor.idProduct);
 
-  if ((result = usb_setup_iterator (&deviceIterator)) < 0)
-    return result;
+  /* set up the matching dictionary for class IOUSBDevice and its subclasses.
+     It will be consumed by the IOServiceGetMatchingServices call */
+  if ((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL) {
+    darwin_cleanup ();
+    
+    USB_ERROR(-ENOMEM);
+  }
 
-  /* This port of libusb uses locations to keep track of devices. */
-  while ((darwin_device = usb_get_next_device (deviceIterator, &dlocation)) != NULL) {
-    if (dlocation == location)
+  result = IOServiceGetMatchingServices(masterPort,
+					matchingDict,
+					&deviceIterator);
+  matchingDict = NULL;
+
+  if (result != kIOReturnSuccess)
+    USB_ERROR_STR(-darwin_to_errno (result), "usb_os_open(IOServiceGetMatchingServices): %s", darwin_error_str(result));
+
+  while (IOIteratorIsValid (deviceIterator) && (usbDevice = IOIteratorNext(deviceIterator))) {
+    /* Create an intermediate plug-in */
+    result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
+                                               kIOCFPlugInInterfaceID, &plugInInterface,
+                                               &score);
+
+    result = IOObjectRelease(usbDevice);
+    if (result || !plugInInterface)
+      continue;
+
+    (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(DeviceInterfaceID),
+                                       (LPVOID)&darwin_device);
+
+    (*plugInInterface)->Stop(plugInInterface);
+    IODestroyPlugInInterface (plugInInterface);
+    plugInInterface = NULL;
+
+    if (!darwin_device)
+      continue;
+
+    result = (*(darwin_device))->GetLocationID(darwin_device, (u_int32_t *)&dlocation);
+    if (dlocation == location) {
+      device->device = darwin_device;
       break;
+    }
 
     (*darwin_device)->Release(darwin_device);
   }
 
   IOObjectRelease(deviceIterator);
-  device->device = darwin_device;
 
   if (device->device == NULL)
     USB_ERROR_STR (-ENOENT, "usb_os_open: %s\n", "Device not found!");
@@ -332,8 +290,7 @@ int usb_os_open(usb_dev_handle *dev)
       break;
     default:
       (*(device->device))->Release (device->device);
-      USB_ERROR_STR(-darwin_to_errno (result), "usb_os_open(USBDeviceOpenSeize): %s",
-		    darwin_error_str(result));
+      USB_ERROR_STR(-darwin_to_errno (result), "usb_os_open(USBDeviceOpenSeize): %s", darwin_error_str(result));
     }
     
     device->open = 0;
@@ -344,9 +301,6 @@ int usb_os_open(usb_dev_handle *dev)
   dev->interface = -1;
   dev->altsetting = -1;
 
-  device->num_endpoints  = 0;
-  device->endpoint_addrs = NULL;
-
   return 0;
 }
 
@@ -384,64 +338,10 @@ int usb_os_close(usb_dev_handle *dev)
   return 0;
 }
 
-static int get_endpoints (struct darwin_dev_handle *device)
-{
-  io_return_t ret;
-
-  u_int8_t numep, direction, number;
-  u_int8_t dont_care1, dont_care3;
-  u_int16_t dont_care2;
-
-  int i;
-
-  if (device == NULL || device->interface == NULL)
-    return -EINVAL;
-
-  if (usb_debug > 1)
-    fprintf(stderr, "libusb/darwin.c get_endpoints: building table of endpoints.\n");
-
-  /* retrieve the total number of endpoints on this interface */
-  ret = (*(device->interface))->GetNumEndpoints(device->interface, &numep);
-  if ( ret ) {
-    if ( usb_debug > 1 )
-      fprintf ( stderr, "get_endpoints: interface is %p\n", device->interface );
-
-    USB_ERROR_STR ( -ret, "get_endpoints: can't get number of endpoints for interface" );
-  }
-
-  free (device->endpoint_addrs);
-  device->endpoint_addrs = calloc (sizeof (unsigned char), numep);
-
-  /* iterate through pipe references */
-  for (i = 1 ; i <= numep ; i++) {
-    ret = (*(device->interface))->GetPipeProperties(device->interface, i, &direction, &number,
-						    &dont_care1, &dont_care2, &dont_care3);
-
-    if (ret != kIOReturnSuccess) {
-      fprintf (stderr, "get_endpoints: an error occurred getting pipe information on pipe %d\n",
-	       i );
-      USB_ERROR_STR(-darwin_to_errno(ret), "get_endpoints(GetPipeProperties): %s", darwin_error_str(ret));
-    }
-
-    if (usb_debug > 1)
-      fprintf (stderr, "get_endpoints: Pipe %i: DIR: %i number: %i\n", i, direction, number);
-
-    device->endpoint_addrs[i - 1] = ((direction << 7 & USB_ENDPOINT_DIR_MASK) |
-				     (number & USB_ENDPOINT_ADDRESS_MASK));
-  }
-
-  device->num_endpoints = numep;
-
-  if (usb_debug > 1)
-    fprintf(stderr, "libusb/darwin.c get_endpoints: complete.\n");
-  
-  return 0;
-}
-
-static int claim_interface (usb_dev_handle *dev, int interface)
+static int claim_interface ( usb_dev_handle *dev, int interface )
 {
   io_iterator_t interface_iterator;
-  io_service_t  usbInterface = IO_OBJECT_NULL;
+  io_service_t  usbInterface;
   io_return_t result;
   io_cf_plugin_ref_t *plugInInterface = NULL;
 
@@ -463,13 +363,12 @@ static int claim_interface (usb_dev_handle *dev, int interface)
     USB_ERROR_STR (-darwin_to_errno(result), "claim_interface(CreateInterfaceIterator): %s",
 		   darwin_error_str(result));
 
-  for ( current_interface=0 ; current_interface <= interface ; current_interface++ ) {
+  for ( current_interface=0; current_interface<=interface; current_interface++ ) {
     usbInterface = IOIteratorNext(interface_iterator);
     if ( usb_debug > 3 )
-      fprintf ( stderr, "Interface %d of device is 0x%08x\n",
+      fprintf ( stderr, "Interface %d of device is 0x%x\n",
 		current_interface, usbInterface );
   }
-
   current_interface--;
 
   /* the interface iterator is no longer needed, release it */
@@ -524,8 +423,6 @@ static int claim_interface (usb_dev_handle *dev, int interface)
 	USB_ERROR_STR(-darwin_to_errno(result), "claim_interface(SetConfiguration): %s",
 		      darwin_error_str(result));
       }
-
-      dev->config = configDesc->bConfigurationValue;
     }
     
     request.bInterfaceClass = kIOUSBFindInterfaceDontCare;
@@ -543,7 +440,7 @@ static int claim_interface (usb_dev_handle *dev, int interface)
       usbInterface = IOIteratorNext(interface_iterator);
 
       if ( usb_debug > 3 )
-	fprintf ( stderr, "claim_interface: Interface %d of device is 0x%08x\n",
+	fprintf ( stderr, "claim_interface: Interface %d of device is 0x%x\n",
 		  current_interface, usbInterface );
     }
     current_interface--;
@@ -581,7 +478,7 @@ static int claim_interface (usb_dev_handle *dev, int interface)
     USB_ERROR(-EACCES);
 
   if ( usb_debug > 3 )
-    fprintf ( stderr, "claim_interface: Interface %d of device from QueryInterface is %p\n",
+    fprintf ( stderr, "claim_interface: Interface %d of device from QueryInterface is 0x%x\n",
 	      current_interface, device->interface);
 
   /* claim the interface */
@@ -590,22 +487,13 @@ static int claim_interface (usb_dev_handle *dev, int interface)
     USB_ERROR_STR(-darwin_to_errno(result), "claim_interface(USBInterfaceOpen): %s",
 		  darwin_error_str(result));
 
-  result = get_endpoints (device);
-
-  if (result) {
-    /* this should not happen */
-    usb_release_interface (dev, interface);
-    USB_ERROR_STR ( result, "claim_interface: could not build endpoint table");
-  }
-
   return 0;
 }
 
-int usb_set_configuration (usb_dev_handle *dev, int configuration)
+int usb_set_configuration(usb_dev_handle *dev, int configuration)
 {
   struct darwin_dev_handle *device;
   io_return_t result;
-  int interface;
 
   if ( usb_debug > 3 )
     fprintf ( stderr, "usb_set_configuration: called for config %x\n", configuration );
@@ -618,8 +506,6 @@ int usb_set_configuration (usb_dev_handle *dev, int configuration)
 
   /* Setting configuration will invalidate the interface, so we need
      to reclaim it. First, dispose of existing interface, if any. */
-  interface = dev->interface;
-
   if ( device->interface )
     usb_release_interface(dev, dev->interface);
 
@@ -629,9 +515,9 @@ int usb_set_configuration (usb_dev_handle *dev, int configuration)
     USB_ERROR_STR(-darwin_to_errno(result), "usb_set_configuration(SetConfiguration): %s",
 		  darwin_error_str(result));
 
-  /* Reclaim interface */
-  if (interface != -1)
-    result = usb_claim_interface (dev, interface);
+  /* Reclaim interface: assume zero */
+  if (dev->interface != -1)
+    result = claim_interface(dev, dev->interface);
 
   dev->config = configuration;
 
@@ -696,11 +582,6 @@ int usb_release_interface(usb_dev_handle *dev, int interface)
 
   device->interface = NULL;
 
-  free (device->endpoint_addrs);
-
-  device->num_endpoints  = 0;
-  device->endpoint_addrs = NULL;
-
   dev->interface = -1;
   dev->altsetting = -1;
 
@@ -729,57 +610,77 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
 
   dev->altsetting = alternate;
 
-  result = get_endpoints (device);
-  if (result) {
-    /* this should not happen */
-    USB_ERROR_STR ( result, "usb_set_altinterface: could not build endpoint table");
-  }
-
   return 0;
 }
 
 /* simple function that figures out what pipeRef is associated with an endpoint */
 static int ep_to_pipeRef (struct darwin_dev_handle *device, int ep)
 {
+  io_return_t ret;
+
+  u_int8_t numep, direction, number;
+  u_int8_t dont_care1, dont_care3;
+  u_int16_t dont_care2;
+
   int i;
 
   if (usb_debug > 1)
-    fprintf(stderr, "libusb/darwin.c ep_to_pipeRef: Converting ep address to pipeRef.\n");
+    fprintf(stderr, "Converting ep address to pipeRef.\n");
 
-  for (i = 0 ; i < device->num_endpoints ; i++)
-    if (device->endpoint_addrs[i] == ep)
-      return i + 1;
+  /* retrieve the total number of endpoints on this interface */
+  ret = (*(device->interface))->GetNumEndpoints(device->interface, &numep);
+  if ( ret ) {
+    if ( usb_debug > 1 )
+      fprintf ( stderr, "ep_to_pipeRef: interface is %x\n", device->interface );
+    USB_ERROR_STR ( -ret, "ep_to_pipeRef: can't get number of endpoints for interface" );
+  }
+
+  /* iterate through the pipeRefs until we find the correct one */
+  for (i = 1 ; i <= numep ; i++) {
+    ret = (*(device->interface))->GetPipeProperties(device->interface, i, &direction, &number,
+						    &dont_care1, &dont_care2, &dont_care3);
+
+    if (ret != kIOReturnSuccess) {
+      fprintf (stderr, "ep_to_pipeRef: an error occurred getting pipe information on pipe %d\n",
+	       i );
+      USB_ERROR_STR(-darwin_to_errno(ret), "ep_to_pipeRef(GetPipeProperties): %s", darwin_error_str(ret));
+    }
+
+    if (usb_debug > 1)
+      fprintf (stderr, "ep_to_pipeRef: Pipe %i: DIR: %i number: %i\n", i, direction, number);
+
+    /* calculate the endpoint of the pipe and check it versus the requested endpoint */
+    if ( ((direction << 7 & USB_ENDPOINT_DIR_MASK) | (number & USB_ENDPOINT_ADDRESS_MASK)) == ep ) {
+      if (usb_debug > 1)
+	fprintf(stderr, "ep_to_pipeRef: pipeRef for ep address 0x%02x found: 0x%02x\n", ep, i);
+
+      return i;
+    }
+  }
 
-  /* No pipe found with the correct endpoint address */
   if (usb_debug > 1)
-    fprintf(stderr, "libusb/darwin.c ep_to_pipeRef: No pipeRef found with endpoint address 0x%02x.\n", ep);
+    fprintf(stderr, "ep_to_pipeRef: No pipeRef found with endpoint address 0x%02x.\n", ep);
   
+  /* none of the found pipes match the requested endpoint */
   return -1;
 }
 
-/* argument to handle multiple parameters to rw_completed */
-struct rw_complete_arg {
-  UInt32        io_size;
-  IOReturn      result;
-  CFRunLoopRef  cf_loop;
-};
-
-static void rw_completed(void *refcon, io_return_t result, void *io_size)
+static void rw_completed(void *ret_io_size, io_return_t result, void *io_size)
 {
-  struct rw_complete_arg *rw_arg = (struct rw_complete_arg *)refcon;
-
   if (usb_debug > 2)
-    fprintf(stderr, "io async operation completed: %s, size=%lu, result=0x%08x\n", darwin_error_str(result),
-	    (UInt32)io_size, result);
+    fprintf(stderr, "io async operation completed. result = %i, size = %i\n", result,
+	    (u_int32_t)io_size);
 
-  rw_arg->io_size = (UInt32)io_size;
-  rw_arg->result  = result;
+  if (result == kIOReturnSuccess)
+    *(u_int32_t *)ret_io_size = (u_int32_t)io_size;
+  else
+    *(u_int32_t *)ret_io_size = result;
 
-  CFRunLoopStop(rw_arg->cf_loop);
+  CFRunLoopStop(CFRunLoopGetCurrent());
 }
 
-static int usb_bulk_transfer (usb_dev_handle *dev, int ep, char *bytes, int size, int timeout,
-			      rw_async_func_t rw_async, rw_async_to_func_t rw_async_to)
+int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
+                   int timeout)
 {
   struct darwin_dev_handle *device;
 
@@ -788,134 +689,131 @@ static int usb_bulk_transfer (usb_dev_handle *dev, int ep, char *bytes, int size
   CFRunLoopSourceRef cfSource;
   int pipeRef;
 
-  struct rw_complete_arg rw_arg;
-
-  u_int8_t  transferType;
-
-  /* None of the values below are used in libusb for bulk transfers */
-  u_int8_t  direction, number, interval;
+  u_int8_t  direction, number, transferType, interval;
   u_int16_t maxPacketSize;
+  u_int32_t written;
 
   if (!dev)
-    USB_ERROR_STR ( -ENXIO, "usb_bulk_transfer: Called with NULL device" );
+    USB_ERROR_STR ( -ENXIO, "usb_bulk_read: called with null device" );
 
   if ((device = dev->impl_info) == NULL)
-    USB_ERROR_STR ( -ENOENT, "usb_bulk_transfer: Device not open" );
+    USB_ERROR_STR ( -ENOENT, "usb_bulk_read: device not initialized" );
 
   /* interface is not open */
   if (!device->interface)
-    USB_ERROR_STR(-EACCES, "usb_bulk_transfer: Interface used before it was opened");
+    USB_ERROR_STR(-EACCES, "usb_bulk_write: interface used without being claimed");
 
-
-  /* Set up transfer */
   if ((pipeRef = ep_to_pipeRef(device, ep)) < 0)
-    USB_ERROR_STR ( -EINVAL, "usb_bulk_transfer: Invalid pipe reference" );
+    USB_ERROR(-EINVAL);
 
-  (*(device->interface))->GetPipeProperties (device->interface, pipeRef, &direction, &number,
-					     &transferType, &maxPacketSize, &interval);
+  if (usb_debug > 3)
+    fprintf(stderr, "usb_bulk_write: endpoint=0x%02x size=%i TO=%i\n", ep, size, timeout);
+
+  (*(device->interface))->GetPipeProperties ( device->interface, pipeRef, &direction, &number,
+					     &transferType, &maxPacketSize, &interval );
 
   (*(device->interface))->CreateInterfaceAsyncEventSource(device->interface, &cfSource);
   CFRunLoopAddSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
 
-  bzero((void *)&rw_arg, sizeof(struct rw_complete_arg));
-  rw_arg.cf_loop = CFRunLoopGetCurrent();
-  /* Transfer set up complete */
-
-  if (usb_debug > 0)
-    fprintf (stderr, "libusb/darwin.c usb_bulk_transfer: Transfering %i bytes of data on endpoint 0x%02x\n",
-	     size, ep);
-
-  /* Bulk transfer */
-  if (transferType == kUSBInterrupt && usb_debug > 3)
-    fprintf (stderr, "libusb/darwin.c usb_bulk_transfer: USB pipe is an interrupt pipe. Timeouts will not be used.\n");
-
-  if ( transferType != kUSBInterrupt && rw_async_to != NULL)
-
-    result = rw_async_to (device->interface, pipeRef, bytes, size, timeout, timeout,
-			  (IOAsyncCallback1)rw_completed, (void *)&rw_arg);
+  /* there seems to be no way to determine how many bytes are actually written */
+#if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
+  if ( transferType != kUSBInterrupt )
+    result = (*(device->interface))->WritePipeAsyncTO(device->interface, pipeRef,
+						      bytes, size, 0, timeout,
+						      (IOAsyncCallback1)rw_completed,
+						      (void *)&written);
   else
-    result = rw_async (device->interface, pipeRef, bytes, size, (IOAsyncCallback1)rw_completed,
-		       (void *)&rw_arg);
-
-  if (result == kIOReturnSuccess) {
+#endif
+    result = (*(device->interface))->WritePipeAsync(device->interface, pipeRef, bytes,
+						    size, (IOAsyncCallback1)rw_completed,
+						    (void *)&written);
+  
+  if (result == kIOReturnSuccess)
     /* wait for write to complete */
-    if (CFRunLoopRunInMode(kCFRunLoopDefaultMode, (timeout+999)/1000, true) == kCFRunLoopRunTimedOut) {
-      (*(device->interface))->AbortPipe(device->interface, pipeRef);
-      CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0, true); /* Pick up aborted callback */
-      if (usb_debug)
-	fprintf(stderr, "usb_bulk_read: input timed out\n");
-    }
-  }
+    CFRunLoopRun();
 
   CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
-  
-  /* Check the return code of both the write and completion functions. */
-  if (result != kIOReturnSuccess || (rw_arg.result != kIOReturnSuccess && 
-      rw_arg.result != kIOReturnAborted) ) {
-    int error_code;
-    char *error_str;
 
-    if (result == kIOReturnSuccess) {
-      error_code = darwin_to_errno (rw_arg.result);
-      error_str  = darwin_error_str (rw_arg.result);
-    } else {
-      error_code = darwin_to_errno(result);
-      error_str  = darwin_error_str (result);
-    }
+  if (result != kIOReturnSuccess)
+    USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_write(WritePipeAsyncTO): %s", darwin_error_str(result));
+
+  return written;
+}
+
+int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
+                  int timeout)
+{
+  struct darwin_dev_handle *device;
+  io_return_t result = -1;
+
+  int pipeRef;
+
+  u_int32_t retrieved = 0;
+  u_int16_t maxPacketSize;
+  u_int8_t  direction, number, transferType, interval;
+
+  CFRunLoopSourceRef cfSource;
+
+  if (usb_debug)
+    fprintf (stderr, "usb_bulk_read: ep addr = 0x%02x\n", ep);
+
+  ep |= 0x80; /* USB_DIR_OUT */
+
+  if (!dev)
+    USB_ERROR_STR ( -ENXIO, "usb_bulk_read: called with null device" );
+
+  if ((device = dev->impl_info) == NULL)
+    USB_ERROR_STR ( -ENOENT, "usb_bulk_read: device not initialized" );
+
+  /* interface is not open */
+  if (!device->interface)
+    USB_ERROR_STR(-EACCES, "usb_bulk_read: interface used without being claimed");
+
+  if ((pipeRef = ep_to_pipeRef(device, ep)) == -1)
+    USB_ERROR_STR ( -EINVAL, "usb_bulk_read: invalid pipe reference" );
+
+  if (usb_debug > 3)
+    fprintf(stderr, "usb_bulk_read: endpoint=0x%02x size=%i timeout=%fsec\n" , ep, size, (double)timeout/1000.0 );
+
+  (*(device->interface))->GetPipeProperties ( device->interface, pipeRef, &direction, &number,
+					     &transferType, &maxPacketSize, &interval );
+
+  (*(device->interface))->CreateInterfaceAsyncEventSource(device->interface, &cfSource);
+  CFRunLoopAddSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
+
+#if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
+  if ( transferType == kUSBInterrupt ) {
+#endif
+    /* This is an interrupt pipe. We can't specify a timeout. */
+    if (usb_debug > 3)
+      fprintf(stderr, "usb_bulk_read: interrupt pipe\n" );
+    result = (*(device->interface))->ReadPipeAsync(device->interface, pipeRef, bytes,
+					      size, (IOAsyncCallback1)rw_completed,
+					      (void *)&retrieved);
+#if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
+  } else {
+    result = (*(device->interface))->ReadPipeAsyncTO(device->interface, pipeRef,
+						     bytes + retrieved, size, 0,
+						     timeout, (IOAsyncCallback1)rw_completed,
+						     (void *)&retrieved);
     
-    if (transferType != kUSBInterrupt && rw_async_to != NULL)
-      USB_ERROR_STR(-error_code, "usb_bulk_transfer (w/ Timeout): %s", error_str);
-    else
-      USB_ERROR_STR(-error_code, "usb_bulk_transfer (No Timeout): %s", error_str);
+    if (result != kIOReturnSuccess)
+      USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_read(ReadPipeTO): error reading from bulk endpoint %02x: %s",
+		    ep, darwin_error_str(result));
   }
-
-  return rw_arg.io_size;
-}
-
-int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size, int timeout)
-{
-  int result;
-  rw_async_to_func_t to_func = NULL;
-  struct darwin_dev_handle *device;
-  
-  if (dev == NULL || dev->impl_info == NULL)
-    return -EINVAL;
-
-  device = dev->impl_info;
-
-#if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
-  to_func = (*(device->interface))->WritePipeAsyncTO;
 #endif
 
-  if ((result = usb_bulk_transfer (dev, ep, bytes, size, timeout,
-				   (*(device->interface))->WritePipeAsync, to_func)) < 0)
-    USB_ERROR_STR (result, "usb_bulk_write: An error occured during write (see messages above)");
-  
-  return result;
-}
+  if (result == kIOReturnSuccess)
+    /* wait for write to complete */
+    CFRunLoopRun();
 
-int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size, int timeout)
-{
-  int result;
-  rw_async_to_func_t to_func = NULL;
-  struct darwin_dev_handle *device;
-  
-  if (dev == NULL || dev->impl_info == NULL)
-    return -EINVAL;
+  CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
 
-  ep |= 0x80;
-  
-  device = dev->impl_info;
+  if (result != kIOReturnSuccess || retrieved < 0)
+    USB_ERROR_STR(-darwin_to_errno(result), "usb_bulk_read(ReadPipe): error reading from bulk endpoint %02x: %s",
+		  ep, darwin_error_str(result));
 
-#if !defined (LIBUSB_NO_TIMEOUT_INTERFACE)
-  to_func = (*(device->interface))->ReadPipeAsyncTO;
-#endif
-
-  if ((result = usb_bulk_transfer (dev, ep, bytes, size, timeout,
-				   (*(device->interface))->ReadPipeAsync, to_func)) < 0)
-    USB_ERROR_STR (result, "usb_bulk_read: An error occured during read (see messages above)");
-  
-  return result;
+  return retrieved;
 }
 
 /* interrupt endpoints seem to be treated just like any other endpoint under OSX/Darwin */
@@ -958,7 +856,6 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
   urequest.pData = bytes;
 #if !defined (LIBUSB_NO_TIMEOUT_DEVICE)
   urequest.completionTimeout = timeout;
-  urequest.noDataTimeout = timeout;
 
   result = (*(device->device))->DeviceRequestTO(device->device, &urequest);
 #else
@@ -976,37 +873,79 @@ int usb_os_find_busses(struct usb_bus **busses)
   struct usb_bus *fbus = NULL;
 
   io_iterator_t deviceIterator;
+  io_service_t usbDevice;
   io_return_t result;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
 
   usb_device_t **device;
 
-  UInt32 location;
+  s_int32_t score;
+  u_int32_t location;
 
   char buf[20];
   int i = 1;
 
   /* Create a master port for communication with IOKit (this should
      have been created if the user called usb_init() )*/
-  if (masterPort == MACH_PORT_NULL) {
+  if (!masterPort) {
     usb_init ();
 
-    if (masterPort == MACH_PORT_NULL)
+    if (!masterPort)
       USB_ERROR(-ENOENT);
   }
 
-  if ((result = usb_setup_iterator (&deviceIterator)) < 0)
-    return result;
+  /* set up the matching dictionary for class IOUSBRootHubDevice
+     and its subclasses. It will be consumed by the next call */
+  if ((matchingDict = IOServiceMatching("IOUSBRootHubDevice")) == NULL) {
+    darwin_cleanup ();
 
-  while ((device = usb_get_next_device (deviceIterator, &location)) != NULL) {
+    USB_ERROR(-ENOMEM);
+  }
+
+  result = IOServiceGetMatchingServices(masterPort,
+					matchingDict,
+					&deviceIterator);
+  matchingDict = NULL;
+
+  if (result != kIOReturnSuccess)
+    USB_ERROR_STR(-darwin_to_errno (result), "usb_os_find_busses(IOServiceGetMatchingServices): %s",
+		  darwin_error_str(result));
+
+
+  while (IOIteratorIsValid(deviceIterator) && (usbDevice = IOIteratorNext(deviceIterator))) {
     struct usb_bus *bus;
 
-    if (location & 0x00ffffff)
+    /* Create an intermediate plug-in */
+    result = IOCreatePlugInInterfaceForService(usbDevice,
+					       kIOUSBDeviceUserClientTypeID,
+                                               kIOCFPlugInInterfaceID,
+					       &plugInInterface,
+                                               &score);
+
+    result = IOObjectRelease(usbDevice);
+    if (result || !plugInInterface)
       continue;
 
-    bus = calloc(1, sizeof(struct usb_bus));
-    if (bus == NULL)
+    (*plugInInterface)->QueryInterface(plugInInterface,
+				       CFUUIDGetUUIDBytes(DeviceInterfaceID),
+                                       (LPVOID)&device);
+
+    /* done with this */
+    (*plugInInterface)->Stop(plugInInterface);
+    IODestroyPlugInInterface (plugInInterface);
+    plugInInterface = NULL;
+
+    if (!device)
+      continue;
+
+    result = (*(device))->GetLocationID(device, &location);
+
+    bus = malloc(sizeof(*bus));
+    if (!bus)
       USB_ERROR(-ENOMEM);
     
+    memset((void *)bus, 0, sizeof(*bus));
+    
     sprintf(buf, "%03i", i++);
     bus->location = location;
 
@@ -1033,54 +972,91 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
   struct usb_device *fdev = NULL;
 
   io_iterator_t deviceIterator;
+  io_service_t usbDevice;
   io_return_t result;
+  io_cf_plugin_ref_t *plugInInterface = NULL;
 
   usb_device_t **device;
 
+  s_int32_t score;
   u_int16_t address;
-  UInt32 location;
-  UInt32 bus_loc = bus->location;
+  u_int32_t location;
+  u_int32_t bus_loc = bus->location;
 
   /* for use in retrieving device description */
   IOUSBDevRequest req;
 
   /* a master port should have been created by usb_os_init */
-  if (masterPort == MACH_PORT_NULL)
+  if (!masterPort)
     USB_ERROR(-ENOENT);
 
-  if ((result = usb_setup_iterator (&deviceIterator)) < 0)
-    return result;
+  /* set up the matching dictionary for class IOUSBDevice and its subclasses.
+     It will be consumed by the next call */
+  if ((matchingDict = IOServiceMatching(kIOUSBDeviceClassName)) == NULL) {
+    darwin_cleanup ();
+
+    USB_ERROR(-ENOMEM);
+  }
+
+  result = IOServiceGetMatchingServices(masterPort, matchingDict, &deviceIterator);
+  matchingDict = NULL;
+
+  if (result != kIOReturnSuccess)
+    USB_ERROR_STR(-darwin_to_errno (result), "usb_os_find_devices(IOServiceGetMatchingServices): %s", darwin_error_str(result));
 
-  /* Set up request for device descriptor */
   req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
   req.bRequest = kUSBRqGetDescriptor;
   req.wValue = kUSBDeviceDesc << 8;
   req.wIndex = 0;
   req.wLength = sizeof(IOUSBDeviceDescriptor);
 
+  while (IOIteratorIsValid (deviceIterator) && (usbDevice = IOIteratorNext(deviceIterator))) {
+    /* Create an intermediate plug-in */
+    result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
+                                               kIOCFPlugInInterfaceID, &plugInInterface,
+                                               &score);
 
-  while ((device = usb_get_next_device (deviceIterator, &location)) != NULL) {
-    unsigned char device_desc[DEVICE_DESC_LENGTH];
+    result = IOObjectRelease(usbDevice);
+    if (result || !plugInInterface)
+      continue;
+
+    (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(DeviceInterfaceID),
+                                       (LPVOID)&device);
+
+    /* done with this */
+    (*plugInInterface)->Stop(plugInInterface);
+    IODestroyPlugInInterface (plugInInterface);
+    plugInInterface = NULL;
+
+    if (!device)
+      continue;
 
     result = (*(device))->GetDeviceAddress(device, (USBDeviceAddress *)&address);
+    result = (*(device))->GetLocationID(device, &location);
 
     if (usb_debug >= 2)
-      fprintf(stderr, "usb_os_find_devices: Found USB device at location 0x%08lx\n", location);
+      fprintf(stderr, "usb_os_find_devices: Found USB device at location 0x%08x\n", location);
 
     /* first byte of location appears to be associated with the device's bus */
     if (location >> 24 == bus_loc >> 24) {
       struct usb_device *dev;
 
-      dev = calloc(1, sizeof(struct usb_device));
-      if (dev == NULL)
+      dev = malloc(sizeof(*dev));
+      if (!dev) {
 	USB_ERROR(-ENOMEM);
+      }
+
+      memset((void *)dev, 0, sizeof(*dev));
 
       dev->bus = bus;
 
-      req.pData = device_desc;
+      req.pData = &(dev->descriptor);
       result = (*(device))->DeviceRequest(device, &req);
 
-      usb_parse_descriptor(device_desc, "bbwbbbbwwwbbbb", &dev->descriptor);
+      USB_LE16_TO_CPU(dev->descriptor.bcdUSB);
+      USB_LE16_TO_CPU(dev->descriptor.idVendor);
+      USB_LE16_TO_CPU(dev->descriptor.idProduct);
+      USB_LE16_TO_CPU(dev->descriptor.bcdDevice);
 
       sprintf(dev->filename, "%03i-%04x-%04x-%02x-%02x", address,
 	      dev->descriptor.idVendor, dev->descriptor.idProduct,
@@ -1092,7 +1068,7 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
       LIST_ADD(fdev, dev);
 
       if (usb_debug >= 2)
-	fprintf(stderr, "usb_os_find_devices: Found %s on %s at location 0x%08lx\n",
+	fprintf(stderr, "usb_os_find_devices: Found %s on %s at location 0x%08x\n",
 		dev->filename, bus->dirname, location);
     }
 
@@ -1115,7 +1091,7 @@ int usb_os_determine_children(struct usb_bus *bus)
 
 void usb_os_init(void)
 {
-  if (masterPort == MACH_PORT_NULL) {
+  if (masterPort == NULL) {
     IOMasterPort(masterPort, &masterPort);
     
     gNotifyPort = IONotificationPortCreate(masterPort);
@@ -1124,7 +1100,7 @@ void usb_os_init(void)
 
 void usb_os_cleanup (void)
 {
-  if (masterPort != MACH_PORT_NULL)
+  if (masterPort != NULL)
     darwin_cleanup ();
 }
 
